@@ -282,6 +282,8 @@ bool nd_ip_to_string(const sockaddr_storage &ip, std::string &dst);
 
 void nd_json_to_string(const nlohmann::json &j,
   std::string &output, bool pretty = false);
+void nd_ordered_json_to_string(const nlohmann::ordered_json &j,
+  std::string &output, bool pretty = false);
 
 uint8_t nd_netmask_to_prefix(const struct sockaddr_storage *netmask);
 uint8_t nd_netmask_to_prefix(const std::string &netmask);
@@ -735,3 +737,6 @@ public:
 };
 
 std::string nd_change_ext(const std::string &filename, const std::string &ext);
+
+bool nd_has_expired(const std::string &tag,
+  time_t ttl, time_t last_update, bool monotonic = true, time_t now = 0);
