@@ -82,6 +82,7 @@
 
 using namespace std;
 using json = nlohmann::json;
+using json_ordered = nlohmann::ordered_json;
 
 std::mutex nd_printf_mutex;
 
@@ -424,6 +425,24 @@ void nd_json_to_string(const json &j, string &output, bool pretty) {
     output = j.dump(pretty ? ND_JSON_INDENT : -1, ' ', true,
       json::error_handler_t::replace);
 
+    // TODO: Make this it's own helper function...
+    vector<pair<regex *, string> >::const_iterator i;
+    for (i = ndGC.privacy_regex.begin();
+         i != ndGC.privacy_regex.end();
+         i++)
+    {
+        string result = regex_replace(output, *((*i).first),
+          (*i).second);
+        if (result.size()) output = result;
+    }
+}
+
+void nd_ordered_json_to_string(
+  const json_ordered &j, string &output, bool pretty) {
+    output = j.dump(pretty ? ND_JSON_INDENT : -1, ' ', true,
+      json::error_handler_t::replace);
+
+    // TODO: Make this it's own helper function...
     vector<pair<regex *, string> >::const_iterator i;
     for (i = ndGC.privacy_regex.begin();
          i != ndGC.privacy_regex.end();
@@ -2215,3 +2234,18 @@ string nd_change_ext(const string &filename, const string &ext) {
 
     return result;
 }
+
+bool nd_has_expired(const string &tag,
+  time_t ttl, time_t last_update, bool monotonic, time_t now) {
+
+    if (last_update == 0) return true;
+
+    time_t delta = (now ? now :
+      (monotonic ? nd_time_monotonic() : time(nullptr))) - last_update;
+    bool expired = (delta > ttl);
+#if 0
+    nd_dprintf("%s: expired (delta: %lus > ttl: %lus)? %s\n", tag.c_str(),
+        delta, ttl, (expired) ? "yes" : "no");
+#endif
+    return expired;
+}
